----------------------------
--      Constants         --
----------------------------

local CASTING_SOUND_FILE = "Interface\\AddOns\\FocusInterruptSounds\\casting.wav"
local INTERRUPTED_SOUND_FILE = "Interface\\AddOns\\FocusInterruptSounds\\interrupted.wav"

------------------------------
--      Initialization      --
------------------------------

FocusInterruptSounds = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

function FocusInterruptSounds:OnInitialize()

end


function FocusInterruptSounds:OnEnable()
	_, className = UnitClass("player");

	-- Only do stuff for registered classes
	if ("WARLOCK" == className
		or "MAGE" == className
		or "SHAMAN" == className
		or "WARRIOR" == className
		or "ROGUE" == className
		or "PRIEST" == className
		or "HUNTER" == className
	) then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		self:PrintMessage("Add-on activated for the class " .. className);
	else
		self:PrintMessage("Add-on disabled for the class " .. className);
	end
end

function FocusInterruptSounds:OnDisable()

end

function FocusInterruptSounds:PrintMessage(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff7fff7fFocusInterruptSounds|r: " .. msg);
end

function FocusInterruptSounds:FBlacklistSpell(spellId, spellName, spellSchool)
	if (0 == bit.band(spellSchool, 0x7E)
		or "Mass Dispel" == spellName
		or "Starfire" == spellName
		or "Immolate" == spellName
		or "Searing Pain" == spellName
		or "Empowered Smite" == spellName
		or "Divine Wrath" == spellName
	) then
		return true;
	end

	return false;
end

function FocusInterruptSounds:FChanneledSpell(spellId, spellName, spellSchool)
	if ("Drain Life" == spellName
		or "Drain Mana" == spellName
		or "Mind Flay" == spellName
		or "Tranquility" == spellName
		or "Arcane Missiles" == spellName
	) then
		return true;
	end

	return false;
end

function FocusInterruptSounds:FIsFocusImmune()
	for i = 1, 40 do
		local buffName, _, _, _, _, _ = UnitBuff("focus", i);
		if ("Divine Shield" == buffName
			or "Divine Protection" == buffName
			or "Nether Protection" == buffName
		) then
			return true;
		end
	end

	return false;
end

function FocusInterruptSounds:FIsPetSpellAvailable(spellName)

	-- Verify that the pet can act (i.e. isn't feared)
	if (not GetPetActionsUsable()) then
		return false;
	end

	-- Verify that the spell isn't on cooldown (also checks existence)
	startTime, _, spellEnabled = GetSpellCooldown(spellName, BOOKTYPE_PET);
	if (startTime ~= 0 or not spellEnabled) then
		return false;
	end

	-- Verify mana/energy
	_, _, _, cost, _, _, _, _, _ = GetSpellInfo(spellName);
	if (UnitMana("playerpet") < cost) then
		return false;
	end

	return true;
end

function FocusInterruptSounds:FIsSpellAvailable(spellName)

	-- Verify that the spell isn't on cooldown
	startTime, _, spellEnabled = GetSpellCooldown(spellName);
	if (startTime ~= 0 or not spellEnabled) then
		return false;
	end

	-- Verify mana/energy
	_, _, _, cost, _, _, _, _, _ = GetSpellInfo(spellName);
	if (UnitMana("player") < cost) then
		return false;
	end

	return true;
end

function FocusInterruptSounds:COMBAT_LOG_EVENT_UNFILTERED(timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, param1, param2, param3, param4, param5, param6, ...)

	-- Play a sound when the Focus starts casting
	if (0 ~= bit.band(sourceFlags, COMBATLOG_OBJECT_REACTION_HOSTILE)
		and 0 ~= bit.band(sourceFlags, COMBATLOG_OBJECT_FOCUS)
		and (("SPELL_CAST_START" == eventType and not self:FBlacklistSpell(param1, param2, param3))
			or ("SPELL_CAST_SUCCESS" == eventType and self:FChanneledSpell(param1, param2, param3)))
		and not self:FIsFocusImmune()
	) then
		_, className = UnitClass("player");

		if (("WARLOCK" == className and self:FIsPetSpellAvailable("Spell Lock"))
			or ("MAGE" == className and self:FIsSpellAvailable("Counterspell"))
			or ("SHAMAN" == className and self:FIsSpellAvailable("Earth Shock"))
			or ("WARRIOR" == className and self:FIsSpellAvailable("Pummel"))
			or ("ROGUE" == className and self:FIsSpellAvailable("Kick"))
			or ("PRIEST" == className and self:FIsSpellAvailable("Silence"))
			or ("HUNTER" == className and self:FIsSpellAvailable("Silencing Shot"))
		) then
			self:PrintMessage(sourceName .. " is casting |cffff4444" .. param2 .. "|r!");
			PlaySoundFile(CASTING_SOUND_FILE);
		end
	end

	-- Play sound when the Focus is interrupted
	if ("SPELL_INTERRUPT" == eventType
		and 0 ~= bit.band(sourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE)
		and 0 ~= bit.band(destFlags, COMBATLOG_OBJECT_REACTION_HOSTILE)
		and 0 ~= bit.band(destFlags, COMBATLOG_OBJECT_FOCUS)
	) then
		self:PrintMessage("You've successfully interrupted |cffaaffff" .. param5 .. "|r.");
		PlaySoundFile(INTERRUPTED_SOUND_FILE);
	end
end
